/************************************************************************/
/* integrate.c                                                          */
/*                                                                      */
/* Integration, executable.                                             */
/*                                                                      */
/* Author: Arve Kylling, NILU, Norway.                                  */ 
/*         a lot taken from spline.c by Bernhard Mayer                  */
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

  
/*  @26c@                

@code{integrate} calculates the integral between limits x_min and x_max
by interpolating the data points (x[i], y[i]) with natural cubic
splines or linear interpolation. x_min and x_max are the minimum
and maximum values of the first column in the input_file. The 
x-values in the first column must be in ascending order.

The different options to @code{integrate} are displayed when executing:

@example
integrate -h
@end example

    @c26@ */


#include <ascii.h>
#include <numeric.h>
#include <getopt.h>


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define PROGRAM "integrate"
#define VERSION "1.0"



static void print_usage (char *name)
{
  fprintf (stderr, "\nusage:   %s input_file\n\n", name);
  fprintf (stderr, "%s calculates the integral between limits x_min and x_max\n", PROGRAM);
  fprintf (stderr, "by interpolating the data points (x[i], y[i]) with natural cubic\n");
  fprintf (stderr, "splines (default) or linear interpolation. x_min and x_max are the minimum\n");
  fprintf (stderr, "and maximum values of the first column in the input_file. The \n");
  fprintf (stderr, "x-values in the first column must be in ascending order.\n\n");
  fprintf (stderr, "Optional arguments:\n");
  fprintf (stderr, "  -l            linear interpolation between data points\n");
  fprintf (stderr, "  -p            turn of printing of messages\n\n");
  fprintf (stderr, "Input-file: ASCII-file, blank delimited\n");
  fprintf (stderr, "              x1  y1  z1  ...\n");
  fprintf (stderr, "              x2  y2  z2  ...\n\n");
  fprintf (stderr, "For example  %s -p ../test/integrate_test.in  will create:\n", PROGRAM);
  fprintf (stderr, "               5.000000e-01    1.000000e+00    5.000000e-01\n");
}




int main (int argc, char **argv)
{
  char filename[FILENAME_MAX+200] = "";
  int print =1;
  int rows=0;
  int status=0;
  int linear=0;
  int k=0;
  double *x=NULL, *y=NULL;
  
  int max_columns=0, min_columns=0;
  double **data=NULL;

  double a, b, ans;
  int c=0;

  while ((c=getopt (argc, argv, "plc?h")) != EOF)  {
    switch(c)  {
    case 'l':
      linear = 1;
      break;
    case 'p':  /* no print messages */  
      print = 0;
      break;   
    case 'h':
      print_usage (argv[0]);
      return (-1);
      break;
    case '?':
      print_usage (argv[0]);
      return (-1);
      break;
    default:
      print_usage (argv[0]);
      return (-1);
    }
  }
      

  /* check number of remaining command line arguments */
  if (argc - optind != 1)  {
    print_usage (argv[0]);
    return -1;
  }
  
  strcpy (filename, argv[optind]);
	  
  /* read input file */
  if ( print ) {
    if ( linear ) {
      fprintf (stderr, "Using linear interpolation\n");
    }
    else {
      fprintf (stderr, "Using qubic spline interpolation\n");
    }
    fprintf (stderr, "Results reported for all columns\n");
    fprintf (stderr, " ... reading data from file \"%s\" ...\n", filename);
  }

  /* read file */
  status = ASCII_file2double (filename, &rows, &max_columns, &min_columns, &data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, filename);
    return status;
  }

  if ( print ) {
    fprintf (stderr, " ... counted %d data points\n", rows);
  }
  if ( rows == 0) { 
    fprintf (stderr, "%d data points, aborting.\n", rows);
    status = 99;
    ans = 0.0;
    printf("%13.6e\n",ans);
    return status;
  }

  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }
  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return 0;
  }

  for (k=1; k<min_columns; k++) {

    x = ASCII_column (data, rows, 0);
    y = ASCII_column (data, rows, k);

    if ( linear ) {
      ans = integrate(x,y,rows);
    }
    else {
      a = x[0];
      b = x[rows-1];
      integrate_spline(x,y,rows, a, b,&ans);
    }
  if ( k ==1 && print ) {
    fprintf (stderr, "Results:\n");
  }

    if ( k > 1 ) {   printf("   "); } 
    printf("%13.6e",ans);
  }
  printf("\n");

  ASCII_free_double (data, rows);
  free(x);
  free(y);
  
  return 0;
}
